﻿using System.Reflection;
using System.Xml;

namespace TestProject;

/// <summary>
/// 이미지 리소스 확장
/// </summary>
[ContentProperty("Source")]
public class ImageResourceExtension : IMarkupExtension<ImageSource>
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Property
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region 소스 - Source

    /// <summary>
    /// 소스
    /// </summary>
    public string Source { set; get; }

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region 값 제공하기 - ProvideValue(serviceProvider)

    /// <summary>
    /// 값 제공하기
    /// </summary>
    /// <param name="serviceProvider">서비스 제공자</param>
    /// <returns>이미지 소스</returns>
    /// <exception cref="XamlParseException"></exception>
    public ImageSource ProvideValue(IServiceProvider serviceProvider)
    {
        if(string.IsNullOrEmpty(Source))
        {
            IXmlLineInfoProvider lineInfoProvider = serviceProvider.GetService(typeof(IXmlLineInfoProvider)) as IXmlLineInfoProvider;

            IXmlLineInfo lineInfo = (lineInfoProvider != null) ? lineInfoProvider.XmlLineInfo : new XmlLineInfo();

            throw new XamlParseException("ImageResourceExtension requires Source property to be set", lineInfo);
        }

        string assemblyName = GetType().GetTypeInfo().Assembly.GetName().Name;

        return ImageSource.FromResource($"{assemblyName}.{Source}", typeof(ImageResourceExtension).GetTypeInfo().Assembly);
    }

    #endregion
    #region 값 제공하기 - IMarkupExtension.ProvideValue(serviceProvider)

    /// <summary>
    /// 값 제공하기
    /// </summary>
    /// <param name="serviceProvider">서비스 제공자</param>
    /// <returns>마크업 확장 인터페이스</returns>
    object IMarkupExtension.ProvideValue(IServiceProvider serviceProvider)
    {
        return (this as IMarkupExtension<ImageSource>).ProvideValue(serviceProvider);
    }

    #endregion
}